/**
  ******************************************************************************
  * @file    BSP/Src/tft_v2.c
  * @author
  * @brief
  *
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 UTIA AV CR, v.v.i.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "tft_v2.h"

/** @addtogroup STM32H7xx_HAL_Examples
  * @{
  */

/** @addtogroup BSP
  * @{
  */

/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/**
  * @brief  Initialize SeeSaw chip on adafruit 1.8" TFT shield V2
  * @param  None
  * @retval None
  */
void TFT_V2_ShieldInit(I2C_HandleTypeDef *I2cHandle)
{
  uint8_t i2c_d [6];

  // reset all seesaw regs to defaults
  i2c_d[0] = 0x00;
  i2c_d[1] = 0x7F;
  i2c_d[2] = 0xFF;
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 3, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }
  HAL_Delay(500);

#if 0
  // read ID
  i2c_d[0] = 0x00;
  i2c_d[1] = 0x01;
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 2, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }
  while(HAL_I2C_Master_Receive(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 1, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }
#endif

  // enable backlight
  i2c_d[0] = 0x08;
  i2c_d[1] = 0x01;
  i2c_d[2] = 0x00;
  i2c_d[3] = 0xFF;
  i2c_d[4] = 0xFF;
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 5, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }

  // set reset pin as output
  i2c_d[0] = 0x01;
  i2c_d[1] = 0x02;
  i2c_d[2] = (uint8_t)(TFTSHIELD_V2_RESET_PIN >> 24);
  i2c_d[3] = (uint8_t)(TFTSHIELD_V2_RESET_PIN >> 16);
  i2c_d[4] = (uint8_t)(TFTSHIELD_V2_RESET_PIN >> 8);
  i2c_d[5] = (uint8_t)(TFTSHIELD_V2_RESET_PIN);
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 6, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }

  // LCD reset
  i2c_d[0] = 0x01;
  i2c_d[1] = 0x05;
  i2c_d[2] = (uint8_t)(TFTSHIELD_V2_RESET_PIN >> 24);
  i2c_d[3] = (uint8_t)(TFTSHIELD_V2_RESET_PIN >> 16);
  i2c_d[4] = (uint8_t)(TFTSHIELD_V2_RESET_PIN >> 8);
  i2c_d[5] = (uint8_t)TFTSHIELD_V2_RESET_PIN;
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 6, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }

  HAL_Delay(20);

  // set seesaw pins as inputs - buttons and JOY
  i2c_d[0] = 0x01;
  i2c_d[1] = 0x03;
  i2c_d[2] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 24);
  i2c_d[3] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 16);
  i2c_d[4] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 8);
  i2c_d[5] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL);
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 6, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }

  // set pin val to high, buttons are active in low level
  i2c_d[0] = 0x01;
  i2c_d[1] = 0x05;
  i2c_d[2] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 24);
  i2c_d[3] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 16);
  i2c_d[4] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 8);
  i2c_d[5] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL);
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 6, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }

  // enable internal PULLUPs
  i2c_d[0] = 0x01;
  i2c_d[1] = 0x0B;
  i2c_d[2] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 24);
  i2c_d[3] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 16);
  i2c_d[4] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL >> 8);
  i2c_d[5] = (uint8_t)(TFTSHIELD_V2_BUTTON_ALL);
  while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 6, 1000) != HAL_OK) {
    if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
      Error_Handler();
    }
  }
}


/**
  * @brief  Check the availability of adafruit 1.8" TFT shield V2 on top of STM32NUCLEO
  *         board. This is done by writing I2C bus. If it fails the shield is not present
  * @param  None
  * @retval SHIELD_DETECTED: 1.8" TFT shield V2 is available
  *         SHIELD_NOT_DETECTED: 1.8" TFT shield V2 is not available
  */
uint8_t TFT_V2_ShieldDetect(I2C_HandleTypeDef *I2cHandle)
{
	uint8_t i2c_d [3];
	uint8_t status = 0;
	uint32_t timeout = 200;

	i2c_d[0] = 0x00;
	i2c_d[1] = 0x7F;
	i2c_d[2] = 0xFF;

	do {
		status = HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 3, 1000);
	} while ( (status != HAL_OK) && (timeout--) );


	if ( status || !timeout ) {
		return SHIELD_V2_NOT_DETECTED;
	}

	return SHIELD_V2_DETECTED;
}

/**
  * @brief  Returns the Joystick key pressed of adafruit 1.8" TFT shield V2.
  * @retval JOYState_TypeDef: Code of the Joystick key pressed.
  */
JOYState_TypeDef TFT_V2_Shield_JOY_GetState(I2C_HandleTypeDef *I2cHandle)
{
	JOYState_TypeDef state;
	uint8_t i2c_d[4];
	uint32_t btn_state;

    i2c_d[0] = 0x01;
    i2c_d[1] = 0x04;

    while(HAL_I2C_Master_Transmit(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 2, 1000) != HAL_OK) {
    	if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
			Error_Handler();
    	}
 	}
	while(HAL_I2C_Master_Receive(I2cHandle, (uint16_t)TFTSHIELD_V2_I2C_ADDR, i2c_d, 4, 1000) != HAL_OK) {
		if (HAL_I2C_GetError(I2cHandle) != HAL_I2C_ERROR_AF) {
			Error_Handler();
		}
	}

    btn_state = ~( (uint32_t)i2c_d[0] << 24 | (uint32_t)i2c_d[1] << 16 | (uint32_t)i2c_d[2] << 8 | (uint32_t)i2c_d[3] );

	if( btn_state & TFTSHIELD_V2_BUTTON_DOWN ) {
	    state = JOY_DOWN;
	} else if ( btn_state & TFTSHIELD_V2_BUTTON_RIGHT ) {
	    state = JOY_RIGHT;
	} else if ( btn_state & TFTSHIELD_V2_BUTTON_IN ) {
		state = JOY_SEL;
	} else if ( btn_state & TFTSHIELD_V2_BUTTON_UP ) {
		state = JOY_UP;
	} else if ( btn_state & TFTSHIELD_V2_BUTTON_LEFT ) {
		state = JOY_LEFT;
	} else {
	    state = JOY_NONE;
	}

	return state;
}

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT UTIA AV CR, v.v.i *****END OF FILE****/
